@@ -1,11 +1,13 @@
 #!/usr/bin/env python
-# Copyright (c) 2015, 2016 Tim Savannah under LGPLv3. See LICENSE (https://gnu.org/licenses/lgpl-3.0.txt) for more information.
+# Copyright (c) 2015, 2016, 2017 Tim Savannah under LGPLv3. See LICENSE (https://gnu.org/licenses/lgpl-3.0.txt) for more information.
 #
 #   Parser implementation
 
 # In general below, all "tag names" (body, div, etc) should be lowercase. The parser will lowercase internally. All attribute names (like `id` in id="123") provided to search functions should be lowercase. Values are not lowercase. This is because doing tons of searches, lowercasing every search can quickly build up. Lowercase it once in your code, not every time you call a function.
 
+import re
 import sys
+import uuid
 
 # Python 2/3 compatibility:
 try:
@@ -24,12 +26,14 @@ from collections import defaultdict
 
 from .constants import IMPLICIT_SELF_CLOSING_TAGS, INVISIBLE_ROOT_TAG, INVISIBLE_ROOT_TAG_START, INVISIBLE_ROOT_TAG_END
 from .exceptions import MultipleRootNodeException
-from .Tags import AdvancedTag, TagCollection
+from .Tags import AdvancedTag, TagCollection, canFilterTags, FilterableTagCollection
 
 import codecs
 
 from .utils import stripIEConditionals, addStartTag
 
+__all__ = ('AdvancedHTMLParser', 'IndexedAdvancedHTMLParser')
+
 class AdvancedHTMLParser(HTMLParser):
     '''
         AdvancedHTMLParser - This class parses and allows searching of  documents
@@ -37,7 +41,7 @@ class AdvancedHTMLParser(HTMLParser):
 
     def __init__(self, filename=None, encoding='utf-8'):
         '''
-            __init__ - Creates an Advanced HTML parser object. For read-only parsing, consider IndexedAdvancedHTMLPaser for faster searching.
+            __init__ - Creates an Advanced HTML parser object. For read-only parsing, consider IndexedAdvancedHTMLParser for faster searching.
 
                 @param filename <str>         - Optional filename to parse. Otherwise use parseFile or parseStr methods.
                 @param encoding <str>         - Specifies the document encoding. Default utf-8
@@ -90,7 +94,7 @@ class AdvancedHTMLParser(HTMLParser):
         if isSelfClosing is False and tagName in IMPLICIT_SELF_CLOSING_TAGS:
             isSelfClosing = True
 
-        newTag = AdvancedTag(tagName, attributeList, isSelfClosing)
+        newTag = AdvancedTag(tagName, attributeList, isSelfClosing, ownerDocument=self)
         if self.root is None:
             self.root = newTag
         elif len(self.inTag) > 0:
@@ -188,8 +192,13 @@ class AdvancedHTMLParser(HTMLParser):
 
     def getRoot(self):
         '''
-            getRoot - returns the root Tag 
-                @return Tag
+            getRoot - returns the root Tag.
+
+              NOTE: if there are multiple roots, this will be a special tag.
+               You may want to consider using getRootNodes instead if this
+               is a possible situation for you.
+
+            @return AdvancedTag
         '''
         return self.root
 
@@ -210,6 +219,22 @@ class AdvancedHTMLParser(HTMLParser):
             return list(root.children)
         return [root]
 
+    def getAllNodes(self):
+        '''
+            getAllNodes - Get every element
+
+            @return TagCollection<AdvancedTag>
+        '''
+
+        ret = TagCollection()
+
+        for rootNode in self.getRootNodes():
+            ret.append(rootNode)
+
+            ret += rootNode.getAllChildNodes()
+
+        return ret
+
     def setRoot(self, root):
         '''
             Sets the root node, and reprocesses the indexes
@@ -251,7 +276,7 @@ class AdvancedHTMLParser(HTMLParser):
             elements.append(root)
 
         for child in root.children:
-            if child.attributes.get('name') == name:
+            if child.getAttribute('name') == name:
                 elements.append(child)
             elements += self.getElementsByName(name, child)
 
@@ -270,7 +295,7 @@ class AdvancedHTMLParser(HTMLParser):
             return root
 
         for child in root.children:
-            if child.attributes.get('id') == _id:
+            if child.getAttribute('id') == _id:
                 return child
             potential = self.getElementById(_id, child)
             if potential is not None:
@@ -313,7 +338,7 @@ class AdvancedHTMLParser(HTMLParser):
             elements.append(root)
 
         for child in root.children:
-            if child.attributes.get(attrName) == attrValue:
+            if child.getAttribute(attrName) == attrValue:
                 elements.append(child)
             elements += self.getElementsByAttr(attrName, attrValue, child)
         return TagCollection(elements)
@@ -357,6 +382,263 @@ class AdvancedHTMLParser(HTMLParser):
             elements += self.getElementsCustomFilter(filterFunc, child)
         return TagCollection(elements)
 
+    def contains(self, em):
+        '''
+            Checks if #em is found anywhere within this element tree
+
+            @param em <AdvancedTag> - Tag of interest
+
+            @return <bool> - If element #em is within this tree
+        '''
+        for rootNode in self.getRootNodes():
+            if rootNode.contains(em):
+                return True
+
+        return False
+
+    __contains__ = contains
+
+    def containsUid(self, uid):
+        '''
+            Check if #uid is found anywhere within this element tree
+
+            @param uid <uuid.UUID> - Uid
+
+            @return <bool> - If #uid is found within this tree
+        '''
+        for rootNode in self.getRootNodes():
+            if rootNode.containsUid(em):
+                return True
+
+        return False
+
+    def __contains__(self, other):
+        if isinstance(other, uuid.UUID):
+            return self.containsUid(other)
+        elif issubclass(other.__class__, AdvancedTag):
+            return self.contains(other)
+        else:
+            raise TypeError('Invalid operand, should be either a uuid.UUID object or an AdvancedTag')
+
+
+    def filter(self, **kwargs):
+        '''
+            filter aka filterAnd - Filter ALL the elements in this DOM.
+
+            Results must match ALL the filter criteria. for ANY, use the *Or methods
+
+            Requires the QueryableList module to be installed (i.e. AdvancedHTMLParser was installed
+              without '--no-deps' flag.)
+            
+            For alternative without QueryableList,
+              consider #AdvancedHTMLParser.AdvancedHTMLParser.find method or the getElement* methods
+
+            Special Keys:
+
+               tagname - The tag name
+               text    - The inner text
+
+            @return TagCollection<AdvancedTag>
+        '''
+        if canFilterTags is False:
+            raise NotImplementedError('filter methods requires QueryableList installed, it is not. Either install QueryableList, or try the less-robust "find" method, or the getElement* methods.')
+
+        allNodes = self.getAllChildNodes() + [self]
+
+        filterableNodes = FilterableTagCollection(allNodes)
+
+        return filterableNodes.filterAnd(**kwargs)
+
+    filterAnd = filter
+
+    def filterOr(self, **kwargs):
+        '''
+            filterOr - Perform a filter operation on this node and all children (and their children, onto the end)
+
+            Results must match ANY the filter criteria. for ALL, use the *AND methods
+
+            For special filter keys, @see #AdvancedHTMLParser.AdvancedHTMLParser.filter
+
+            Requires the QueryableList module to be installed (i.e. AdvancedHTMLParser was installed
+              without '--no-deps' flag.)
+            
+            For alternative, consider AdvancedHTMLParser.AdvancedHTMLParser.find method or the getElement* methods
+
+            @return TagCollection<AdvancedTag>
+        '''
+        if canFilterTags is False:
+            raise NotImplementedError('filter methods requires QueryableList installed, it is not. Either install QueryableList, or try the less-robust "find" method, or the getElement* methods.')
+
+        allNodes = self.getAllNodes() + [self]
+
+        filterableNodes = FilterableTagCollection(allNodes)
+
+        return filterableNodes.filterOr(**kwargs)
+
+    def find(self, **kwargs):
+        '''
+            find - Perform a search of elements using attributes as keys and potential values as values
+            
+               (i.e.  parser.find(name='blah', tagname='span')  will return all elements in this document
+                 with the name "blah" of the tag type "span" )
+
+            Arguments are key = value, or key can equal a tuple/list of values to match ANY of those values.
+
+            Append a key with __contains to test if some strs (or several possible strs) are within an element
+            Append a key with __icontains to perform the same __contains op, but ignoring case
+
+            Special keys:
+
+               tagname    - The tag name of the element
+               text       - The text within an element
+
+            NOTE: Empty string means both "not set" and "no value" in this implementation.
+
+            NOTE: If you installed the QueryableList module (i.e. ran setup.py without --no-deps) it is
+              better to use the "filter"/"filterAnd" or "filterOr" methods, which are also available
+              on all tags and tag collections (tag collections also have filterAllAnd and filterAllOr)
+
+
+            @return TagCollection<AdvancedTag> - A list of tags that matched the filter criteria
+        '''
+
+        if not kwargs:
+            return TagCollection()
+
+
+        # Because of how closures work in python, need a function to generate these lambdas
+        #  because the closure basically references "current key in iteration" and not
+        #  "actual instance" of variable. Seems to me to be a bug... but whatever
+        def _makeTagnameLambda(tagName):
+            return lambda em : em.tagName == tagName
+
+        def _makeAttributeLambda(_key, _value):
+            return lambda em : em.getAttribute(_key, '') == _value
+
+        def _makeTagnameInLambda(tagNames):
+            return lambda em : em.tagName in tagNames
+
+        def _makeAttributeInLambda(_key, _values):
+            return lambda em : em.getAttribute(_key, '') in _values
+
+        def _makeTextLambda(_value):
+            return lambda em : em.text == _value
+
+        def _makeTextInLambda(_values):
+            return lambda em : em.text in _values
+
+        def _makeAttributeContainsLambda(_key, _value, icontains=False):
+            if icontains is False:
+                return lambda em : _value in em.getAttribute(_key, '')
+            else:
+                _value = _value.lower()
+                return lambda em : _value in em.getAttribute(_key, '').lower()
+
+        def _makeTextContainsLambda(_value, icontains=False):
+            if icontains is False:
+                return lambda em : _value in em.text
+            else:
+                _value = _value.lower()
+                return lambda em : _value in em.text.lower()
+
+        def _makeAttributeContainsInLambda(_key, _values, icontains=False):
+            if icontains:
+                _values = tuple([x.lower() for x in _values])
+
+            def _testFunc(em):
+                attrValue = em.getAttribute(_key, '')
+                if icontains:
+                    attrValue = attrValue.lower()
+
+                for value in _values:
+                    if value in attrValue:
+                        return True
+
+                return False
+
+            return _testFunc
+
+        def _makeTextContainsInLambda(_values, icontains=False):
+            if icontains:
+                _values = tuple([x.lower() for x in _values])
+
+            def _testFunc(em):
+                import pdb; pdb.set_trace()
+                text = em.text
+                if icontains:
+                    text = text.lower()
+
+                for value in _values:
+                    if value in text:
+                        return True
+
+                return False
+
+            return _testFunc
+
+        # This will hold all the functions we will chain for matching
+        matchFunctions = []
+
+        # Iterate over all the filter portions, and build a filter.
+        for key, value in kwargs.items():
+            key = key.lower()
+
+            endsIContains = key.endswith('__icontains')
+            endsContains = key.endswith('__contains')
+
+            isValueList = isinstance(value, (list, tuple))
+
+            thisFunc = None
+
+            if endsIContains or endsContains:
+                key = re.sub('__[i]{0,1}contains$', '', key)
+                if key == 'tagname':
+                    raise ValueError('tagname is not supported for contains')
+
+                if isValueList:
+                    if key == 'text':
+                        thisFunc = _makeTextContainsInLambda(value, icontains=endsIContains)
+                    else:
+                        thisFunc = _makeAttributeContainsLambda(key, value, icontains=endsIContains)
+                else:
+                    if key == 'text':
+                        thisFunc = _makeTextContainsLambda(value, icontains=endsIContains)
+                    else:
+                        thisFunc = _makeAttributeContainsLambda(key, value, icontains=endsIContains)
+
+            else:
+                # Not contains, straight up
+
+                if isValueList:
+                    if key == 'tagname':
+                        thisFunc = _makeTagnameInLambda(value)
+                    elif key == 'text':
+                        thisFunc = _makeTextInLambda(value)
+                    else:
+                        thisFunc = _makeAttributeInLambda(key, value)
+                else:
+                    if key == 'tagname':
+                        thisFunc = _makeTagnameLambda(value)
+                    elif key == 'text':
+                        thisFunc = _makeTextLambda(value)
+                    else:
+                        thisFunc = _makeAttributeLambda(key, value)
+
+
+            matchFunctions.append( thisFunc )
+
+        # The actual matching function - This will run through the assembled
+        #  #matchFunctions list, testing the element against each match
+        #  and returning all elements in a TagCollection that match this list.
+        def doMatchFunc(em):
+            for matchFunction in matchFunctions:
+                if matchFunction(em) is False:
+                    return False
+
+            return True
+
+        return self.getElementsCustomFilter(doMatchFunc)
+
 
     def getHTML(self):
         '''
@@ -448,6 +730,7 @@ class AdvancedHTMLParser(HTMLParser):
         else:
             self.feed(html)
 
+
 class IndexedAdvancedHTMLParser(AdvancedHTMLParser):
     '''
         An AdvancedHTMLParser that indexes for much much faster searching. If you are doing searching/validation, this is your bet.
