@@ -4,17 +4,17 @@
 # In general below, all "tag names" (body, div, etc) should be lowercase. The parser will lowercase internally. All attribute names (like `id` in id="123") provided to search functions should be lowercase. Values are not lowercase. This is because doing tons of searches, lowercasing every search can quickly build up. Lowercase it once in your code, not every time you call a function.
 
 from .Parser import AdvancedHTMLParser, IndexedAdvancedHTMLParser
-from .Tags import AdvancedTag, TagCollection
+from .Tags import AdvancedTag, TagCollection, toggleAttributesDOM
 from .Formatter import AdvancedHTMLFormatter
 from .Validator import ValidatingAdvancedHTMLParser
 from .exceptions import InvalidCloseException, MissedCloseException, HTMLValidationException
 from .SpecialAttributes import StyleAttribute
 
-__version__ = '6.7.0'
-__version_tuple__ = ('6', '7', '0')
-__int_version_tuple__ = (6, 7, 0)
+__version__ = '7.0.0'
+__version_tuple__ = ('7', '0', '0')
+__int_version_tuple__ = (7, 0, 0)
 
-__all__ = ['AdvancedHTMLParser', 'IndexedAdvancedHTMLParser', 'AdvancedHTMLFormatter', 'AdvancedTag', 'TagCollection'
-    'ValidatingAdvancedHTMLParser', 'MissedCloseException', 'InvalidCloseException', 'HTMLValidationException', 'StyleAttribute' ]
+__all__ = ( 'AdvancedHTMLParser', 'IndexedAdvancedHTMLParser', 'AdvancedHTMLFormatter', 'AdvancedTag', 'TagCollection',
+    'ValidatingAdvancedHTMLParser', 'MissedCloseException', 'InvalidCloseException', 'HTMLValidationException', 'StyleAttribute', 'toggleAttributesDOM' )
 
 #vim: set ts=4 sw=4 expandtab
